'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var deepcopy = /*#__PURE__*/require('deepcopy');
var State;
(function (State) {
  State[State["New"] = 0] = "New";
  State[State["Learning"] = 1] = "Learning";
  State[State["Review"] = 2] = "Review";
  State[State["Relearning"] = 3] = "Relearning";
})(State || (State = {}));
var Rating;
(function (Rating) {
  Rating[Rating["Again"] = 0] = "Again";
  Rating[Rating["Hard"] = 1] = "Hard";
  Rating[Rating["Good"] = 2] = "Good";
  Rating[Rating["Easy"] = 3] = "Easy";
})(Rating || (Rating = {}));
var ReviewLog = function ReviewLog(rating, elapsed_days, scheduled_days, review, state) {
  this.rating = rating;
  this.elapsed_days = elapsed_days;
  this.scheduled_days = scheduled_days;
  this.review = review;
  this.state = state;
};
var Card = /*#__PURE__*/function () {
  function Card() {
    this.due = new Date();
    this.stability = 0;
    this.difficulty = 0;
    this.elapsed_days = 0;
    this.scheduled_days = 0;
    this.reps = 0;
    this.lapses = 0;
    this.state = State.New;
    this.last_review = new Date();
  }
  var _proto = Card.prototype;
  _proto.get_retrievability = function get_retrievability(now) {
    if (this.state == State.Review) {
      var elapsed_days = Math.max(0, Math.floor((now.getTime() - this.last_review.getTime()) / 86400000));
      return Math.exp(Math.log(0.9) * elapsed_days / this.stability);
    } else {
      return null;
    }
  };
  return Card;
}();
var SchedulingInfo = function SchedulingInfo(card, review_log) {
  this.card = card;
  this.review_log = review_log;
};
var SchedulingCards = /*#__PURE__*/function () {
  function SchedulingCards(card) {
    this.again = deepcopy(card);
    this.hard = deepcopy(card);
    this.good = deepcopy(card);
    this.easy = deepcopy(card);
  }
  var _proto2 = SchedulingCards.prototype;
  _proto2.update_state = function update_state(state) {
    if (state == State.New) {
      this.again.state = State.Learning;
      this.hard.state = State.Learning;
      this.good.state = State.Learning;
      this.easy.state = State.Review;
      this.again.lapses += 1;
    } else if (state == State.Learning || state == State.Relearning) {
      this.again.state = state;
      this.hard.state = state;
      this.good.state = State.Review;
      this.easy.state = State.Review;
    } else if (state == State.Review) {
      this.again.state = State.Relearning;
      this.hard.state = State.Review;
      this.good.state = State.Review;
      this.easy.state = State.Review;
      this.again.lapses += 1;
    }
  };
  _proto2.schedule = function schedule(now, hard_interval, good_interval, easy_interval) {
    this.again.scheduled_days = 0;
    this.hard.scheduled_days = hard_interval;
    this.good.scheduled_days = good_interval;
    this.easy.scheduled_days = easy_interval;
    this.again.due = new Date(now.getTime() + 5 * 60 * 1000);
    if (hard_interval > 0) {
      this.hard.due = new Date(now.getTime() + hard_interval * 24 * 60 * 60 * 1000);
    } else {
      this.hard.due = new Date(now.getTime() + 10 * 60 * 1000);
    }
    this.good.due = new Date(now.getTime() + good_interval * 24 * 60 * 60 * 1000);
    this.easy.due = new Date(now.getTime() + easy_interval * 24 * 60 * 60 * 1000);
  };
  _proto2.record_log = function record_log(card, now) {
    var _ref;
    return _ref = {}, _ref[Rating.Again] = new SchedulingInfo(this.again, new ReviewLog(Rating.Again, this.again.scheduled_days, card.elapsed_days, now, card.state)), _ref[Rating.Hard] = new SchedulingInfo(this.hard, new ReviewLog(Rating.Hard, this.hard.scheduled_days, card.elapsed_days, now, card.state)), _ref[Rating.Good] = new SchedulingInfo(this.good, new ReviewLog(Rating.Good, this.good.scheduled_days, card.elapsed_days, now, card.state)), _ref[Rating.Easy] = new SchedulingInfo(this.easy, new ReviewLog(Rating.Easy, this.easy.scheduled_days, card.elapsed_days, now, card.state)), _ref;
  };
  return SchedulingCards;
}();
var Params = function Params() {
  this.request_retention = 0.9;
  this.maximum_interval = 36500;
  this.easy_bonus = 1.3;
  this.hard_factor = 1.2;
  this.w = [1.0, 1.0, 5.0, -0.5, -0.5, 0.2, 1.4, -0.12, 0.8, 2.0, -0.2, 0.2, 1.0];
};
var FSRS = /*#__PURE__*/function () {
  function FSRS() {
    this.p = new Params();
  }
  var _proto3 = FSRS.prototype;
  _proto3.repeat = function repeat(card, now) {
    card = deepcopy(card);
    if (card.state == State.New) {
      card.elapsed_days = 0;
    } else {
      card.elapsed_days = (now.getTime() - card.last_review.getTime()) / 86400000;
    }
    card.last_review = now;
    card.reps += 1;
    var s = new SchedulingCards(card);
    s.update_state(card.state);
    if (card.state == State.New) {
      this.init_ds(s);
      s.again.due = new Date(now.getTime() + 60 * 1000);
      s.hard.due = new Date(now.getTime() + 5 * 60 * 1000);
      s.good.due = new Date(now.getTime() + 10 * 60 * 1000);
      var easy_interval = this.next_interval(s.easy.stability * this.p.easy_bonus);
      s.easy.scheduled_days = easy_interval;
      s.easy.due = new Date(now.getTime() + easy_interval * 24 * 60 * 60 * 1000);
    } else if (card.state == State.Learning || card.state == State.Relearning) {
      var hard_interval = 0;
      var good_interval = this.next_interval(s.good.stability);
      var _easy_interval = Math.max(this.next_interval(s.easy.stability * this.p.easy_bonus), good_interval + 1);
      s.schedule(now, hard_interval, good_interval, _easy_interval);
    } else if (card.state == State.Review) {
      var interval = card.elapsed_days;
      var last_d = card.difficulty;
      var last_s = card.stability;
      var retrievability = Math.exp(Math.log(0.9) * (interval / last_s));
      this.next_ds(s, last_d, last_s, retrievability);
      var _hard_interval = this.next_interval(last_s * this.p.hard_factor);
      var _good_interval = this.next_interval(s.good.stability);
      _hard_interval = Math.min(_hard_interval, _good_interval);
      _good_interval = Math.max(_good_interval, _hard_interval + 1);
      var _easy_interval2 = Math.max(this.next_interval(s.easy.stability * this.p.easy_bonus), _good_interval + 1);
      s.schedule(now, _hard_interval, _good_interval, _easy_interval2);
    }
    return s.record_log(card, now);
  };
  _proto3.init_ds = function init_ds(s) {
    s.again.difficulty = this.init_difficulty(Rating.Again);
    s.again.stability = this.init_stability(Rating.Again);
    s.hard.difficulty = this.init_difficulty(Rating.Hard);
    s.hard.stability = this.init_stability(Rating.Hard);
    s.good.difficulty = this.init_difficulty(Rating.Good);
    s.good.stability = this.init_stability(Rating.Good);
    s.easy.difficulty = this.init_difficulty(Rating.Easy);
    s.easy.stability = this.init_stability(Rating.Easy);
  };
  _proto3.next_ds = function next_ds(s, last_d, last_s, retrievability) {
    s.again.difficulty = this.next_difficulty(last_d, Rating.Again);
    s.again.stability = this.next_forget_stability(s.again.difficulty, last_s, retrievability);
    s.hard.difficulty = this.next_difficulty(last_d, Rating.Hard);
    s.hard.stability = this.next_recall_stability(s.hard.difficulty, last_s, retrievability);
    s.good.difficulty = this.next_difficulty(last_d, Rating.Good);
    s.good.stability = this.next_recall_stability(s.good.difficulty, last_s, retrievability);
    s.easy.difficulty = this.next_difficulty(last_d, Rating.Easy);
    s.easy.stability = this.next_recall_stability(s.easy.difficulty, last_s, retrievability);
  };
  _proto3.init_stability = function init_stability(r) {
    return Math.max(this.p.w[0] + this.p.w[1] * r, 0.1);
  };
  _proto3.init_difficulty = function init_difficulty(r) {
    return Math.min(Math.max(this.p.w[2] + this.p.w[3] * (r - 2), 1), 10);
  };
  _proto3.next_interval = function next_interval(s) {
    var interval = s * Math.log(this.p.request_retention) / Math.log(0.9);
    return Math.min(Math.max(Math.round(interval), 1), this.p.maximum_interval);
  };
  _proto3.next_difficulty = function next_difficulty(d, r) {
    var next_d = d + this.p.w[4] * (r - 2);
    return Math.min(Math.max(this.mean_reversion(this.p.w[2], next_d), 1), 10);
  };
  _proto3.mean_reversion = function mean_reversion(init, current) {
    return this.p.w[5] * init + (1 - this.p.w[5]) * current;
  };
  _proto3.next_recall_stability = function next_recall_stability(d, s, r) {
    return s * (1 + Math.exp(this.p.w[6]) * (11 - d) * Math.pow(s, this.p.w[7]) * (Math.exp((1 - r) * this.p.w[8]) - 1));
  };
  _proto3.next_forget_stability = function next_forget_stability(d, s, r) {
    return this.p.w[9] * Math.pow(d, this.p.w[10]) * Math.pow(s, this.p.w[11]) * Math.exp((1 - r) * this.p.w[12]);
  };
  return FSRS;
}();

exports.Card = Card;
exports.FSRS = FSRS;
//# sourceMappingURL=fsrs.js.cjs.development.js.map
